package featureide_ui.editors;

import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.EventListener;
import java.util.List;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Composite;

import featureide_core.model.FeatureModel;
import featureide_core.model.configuration.Configuration;
import featureide_core.model.configuration.SelectableFeature;
import featureide_core.model.configuration.Selection;
import featureide_ui.editors.configuration.ConfigurationContentProvider;
import featureide_ui.editors.configuration.ConfigurationLabelProvider;

/**
 * requires a configuration as input (via setInput) and additionally a feature
 * model with the constructor
 * 
 * @author ckaestne
 * 
 */
public class ConfigurationTreeViewer extends TreeViewer {

	private FeatureModel featureModel;

	public ConfigurationTreeViewer(Composite parent, int style,
			FeatureModel model) {
		super(parent, style);
		this.featureModel = model;
		setContentProvider(new ConfigurationContentProvider());
		setLabelProvider(new ConfigurationLabelProvider());
		addDoubleClickListener(listener);
	}

	private IDoubleClickListener listener = new IDoubleClickListener() {

		public void doubleClick(DoubleClickEvent event) {
			Object object = ((ITreeSelection) event.getSelection())
					.getFirstElement();
			if (object instanceof SelectableFeature) {
				final SelectableFeature feature = (SelectableFeature) object;
				if (feature.getAutomatic() == Selection.UNDEFINED) {
					// set to the next value
					if (feature.getManual() == Selection.UNDEFINED)
						set(feature, Selection.SELECTED);
					else if (feature.getManual() == Selection.SELECTED)
						set(feature, Selection.UNSELECTED);
					else
						// case: unselected
						set(feature, Selection.UNDEFINED);
					fireChanged();
					ConfigurationTreeViewer.this.refresh();
				}
			}
		}

		private void set(SelectableFeature feature, Selection selection) {
			assert getInput() instanceof Configuration;
			((Configuration) getInput()).setManual(feature, selection);
		}

	};

	protected void fireChanged() {
		for (PropertyChangeListener l : listeners)
			l.propertyChange(null);
	}

	private final List<PropertyChangeListener> listeners = new ArrayList<PropertyChangeListener>();

	public void addChangeListener(PropertyChangeListener listener) {
		listeners.add(listener);
	}

	public void removeChangeListener(PropertyChangeListener listener) {
		listeners.remove(listener);
	}

}
