package featureide_ui.editors;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.ui.parts.GraphicalViewerImpl;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.text.IDocument;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.IEditorActionBarContributor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.ide.IGotoMarker;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.texteditor.IDocumentProvider;
import org.eclipse.ui.texteditor.ITextEditor;

import featureide_core.model.FeatureModel;
import featureide_core.model.GrammarFile;
import featureide_core.model.PropertyConstants;
import featureide_core.model.io.ModelWarning;
import featureide_core.model.io.UnsupportedModelException;
import featureide_core.model.io.guidsl.FeatureModelReader;
import featureide_core.model.io.guidsl.FeatureModelWriter;
import featureide_ui.editors.grammar.GUIDefaults;
import featureide_ui.editors.grammar.GrammarEditorContributor;
import featureide_ui.editors.grammar.actions.AndAction;
import featureide_ui.editors.grammar.actions.CreateCompoundAction;
import featureide_ui.editors.grammar.actions.CreateLayerAction;
import featureide_ui.editors.grammar.actions.DeleteAction;
import featureide_ui.editors.grammar.actions.AlternativeAction;
import featureide_ui.editors.grammar.actions.MandantoryAction;
import featureide_ui.editors.grammar.actions.OrAction;
import featureide_ui.editors.grammar.editparts.GraphicalEditPartFactory;
import featureide_ui.editors.grammar.layouts.FeatureLayoutManager;
import featureide_ui.editors.grammar.layouts.LevelOrderLayout;

/**
 * A multi page editor to edit the model file. If the model file contains errors
 * markers will be created if the user tries to change the page or saves the
 * editors content.
 * 
 * @author Thomas Thm
 * 
 * TODO undo and redo in feature diagram TODO implement hide functionality on
 * open command
 */
public class GrammarEditor extends MultiPageEditorPart implements GUIDefaults,
		PropertyConstants, PropertyChangeListener {

	private GraphicalViewerImpl graphicalViewer;

	private TextEditor textEditor;

	private int graphicalViewerIndex;

	private int textEditorIndex;

	private boolean isPageModified;

	// private IFeatureProject featureProject;

	private FeatureModel featureModel;

	private FeatureModelReader featureModelReader;

	private FeatureModelWriter featureModelWriter;

	private CreateLayerAction createLayerAction;

	private CreateCompoundAction createCompoundAction;

	private DeleteAction deleteAction;

	private MandantoryAction mandantoryAction;

	private AndAction andAction;

	private OrAction orAction;

	private AlternativeAction alternativeAction;

	private FeatureLayoutManager layoutManager = new LevelOrderLayout();

	private GrammarFile grammarFile;

	@Override
	protected void setInput(IEditorInput input) {
		IFile file = (IFile) input.getAdapter(IFile.class);
		grammarFile = new GrammarFile(file);
		// featureProject = CorePlugin.getProjectData(file);
		// setPartName(featureProject.getProjectName() + " Model");
		setPartName(file.getProject().getName() + " Model");
		setTitleToolTip(input.getToolTipText());
		super.setInput(input);

		featureModel = new FeatureModel();
		featureModel.addListener(this);
		featureModelReader = new FeatureModelReader(featureModel);
		featureModelWriter = new FeatureModelWriter(featureModel);

		originalFeatureModel = new FeatureModel();
		try {
			new FeatureModelReader(originalFeatureModel).loadFromFile(file);
		} catch (Exception e) {
		}
	}

	private FeatureModel originalFeatureModel;

	public FeatureModel getOriginalFeatureModel() {
		return originalFeatureModel;
	}

	@Override
	protected void createPages() {
		createDiagramPage();
		createSourcePage();
		createActions();
		createContextMenu();
	}

	void createDiagramPage() {
		graphicalViewer = new ScrollingGraphicalViewer();
		graphicalViewer.setKeyHandler(new GraphicalViewerKeyHandler(
				graphicalViewer));

		graphicalViewer.createControl(getContainer());
		initializeGraphicalViewer();

		graphicalViewer.setEditDomain(new DefaultEditDomain(this));

		initDiagramContent();

		graphicalViewerIndex = addPage(graphicalViewer.getControl());
		setPageText(graphicalViewerIndex, "Feature Diagram");
	}

	void createSourcePage() {
		textEditor = new TextEditor();
		try {
			textEditorIndex = addPage(textEditor, getEditorInput());
			setPageText(textEditorIndex, "Source");
		} catch (PartInitException e) {
			e.printStackTrace();
		}
	}

	private void createActions() {
		createLayerAction = new CreateLayerAction(graphicalViewer, featureModel);
		createCompoundAction = new CreateCompoundAction(graphicalViewer,
				featureModel);
		deleteAction = new DeleteAction(graphicalViewer, featureModel);
		mandantoryAction = new MandantoryAction(graphicalViewer, featureModel);
		andAction = new AndAction(graphicalViewer, featureModel);
		orAction = new OrAction(graphicalViewer, featureModel);
		alternativeAction = new AlternativeAction(graphicalViewer, featureModel);
	}

	private void createContextMenu() {
		MenuManager menu = new MenuManager("#PopupMenu");
		menu.setRemoveAllWhenShown(true);
		menu.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				GrammarEditor.this.fillContextMenu(manager);
			}
		});
		menu.createContextMenu(graphicalViewer.getControl());
		graphicalViewer.setContextMenu(menu);
		getSite().registerContextMenu(menu, graphicalViewer);
	}

	private void fillContextMenu(IMenuManager menu) {
		if (andAction.isEnabled() || orAction.isEnabled()
				|| alternativeAction.isEnabled()) {
			// menu.add(new Separator("featureconnection"));
			menu.add(andAction);
			menu.add(orAction);
			menu.add(alternativeAction);
		} else {
			menu.add(createLayerAction);
			menu.add(createCompoundAction);
			menu.add(deleteAction);
			menu.add(mandantoryAction);
		}
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	public IAction getDiagramAction(String workbenchActionID) {
		if (CreateLayerAction.ID.equals(workbenchActionID))
			return createLayerAction;
		if (CreateCompoundAction.ID.equals(workbenchActionID))
			return createCompoundAction;
		if (DeleteAction.ID.equals(workbenchActionID))
			return deleteAction;
		if (MandantoryAction.ID.equals(workbenchActionID))
			return mandantoryAction;
		if (AndAction.ID.equals(workbenchActionID))
			return andAction;
		if (OrAction.ID.equals(workbenchActionID))
			return orAction;
		if (AlternativeAction.ID.equals(workbenchActionID))
			return alternativeAction;
		return null;
	}

	void initDiagramContent() {
		graphicalViewer.getControl().getDisplay().asyncExec(new Runnable() {
			public void run() {
				graphicalViewer.setContents(featureModel);
				isPageModified = true;
				pageChange(graphicalViewerIndex);
			}
		});
	}

	boolean updateDiagramFromTextEditor() {
		IDocumentProvider provider = textEditor.getDocumentProvider();
		IDocument document = provider.getDocument(textEditor.getEditorInput());
		String text = document.get();

		grammarFile.deleteAllModelMarkers();
		try {
			featureModelReader.readFromString(text);
			for (ModelWarning warning : featureModelReader.getWarnings())
				grammarFile.createModelMarker(warning.message,
						IMarker.SEVERITY_WARNING, warning.line);
		} catch (UnsupportedModelException e) {
			grammarFile.createModelMarker(e.getMessage(),
					IMarker.SEVERITY_ERROR, e.lineNumber);
			return false;
		}
		return true;
	}

	void initializeGraphicalViewer() {
		graphicalViewer.getControl().setBackground(DIAGRAM_BACKGROUND);
		graphicalViewer.setEditPartFactory(new GraphicalEditPartFactory());
		ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
		((ConnectionLayer) rootEditPart
				.getLayer(LayerConstants.CONNECTION_LAYER))
				.setAntialias(SWT.ON);
		graphicalViewer.setRootEditPart(rootEditPart);
	}

	void updateTextEditorFromDiagram() {
		String text = featureModelWriter.writeIntoString();
		IDocumentProvider provider = textEditor.getDocumentProvider();
		IDocument document = provider.getDocument(textEditor.getEditorInput());
		document.set(text);
	}

	public void diagramModified() {
		if (isPageModified)
			return;
		boolean wasDirty = isDirty();
		isPageModified = true;
		if (!wasDirty)
			firePropertyChange(IEditorPart.PROP_DIRTY);
	}

	@Override
	protected void handlePropertyChange(int propertyId) {
		if (propertyId == IEditorPart.PROP_DIRTY)
			isPageModified = isDirty();
		super.handlePropertyChange(propertyId);
	}

	@Override
	public boolean isDirty() {
		return isPageModified || super.isDirty();
	}

	@Override
	public void setFocus() {
		if (getActivePage() == graphicalViewerIndex)
			graphicalViewer.getControl().setFocus();
		else
			textEditor.setFocus();
	}

	@Override
	protected void pageChange(int newPageIndex) {
		if (newPageIndex == textEditorIndex) {
			if (isPageModified)
				updateTextEditorFromDiagram();
		} else { // newPageIndex == graphicalViewerIndex
			if (isDirty() || grammarFile.hasModelMarkers())
				if (!updateDiagramFromTextEditor()) {
					// there are errors in the file, stay at this editor page
					isPageModified = false;
					setActivePage(textEditorIndex);
					return;
				}
		}
		isPageModified = false;

		IEditorActionBarContributor contributor = getEditorSite()
				.getActionBarContributor();
		if (contributor instanceof GrammarEditorContributor)
			((GrammarEditorContributor) contributor).setActivePage(this,
					newPageIndex);

		super.pageChange(newPageIndex);
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
		if (getActivePage() == graphicalViewerIndex && isPageModified) {
			updateTextEditorFromDiagram();
			setActivePage(textEditorIndex);
			setActivePage(graphicalViewerIndex);
		} else
			updateDiagramFromTextEditor();
		isPageModified = false;
		featureModel.performRenamings();
		textEditor.doSave(monitor);
		try {
			new FeatureModelReader(originalFeatureModel).loadFromFile(grammarFile.getResource());
		} catch (Exception e) {
		}
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void doSaveAs() {
		// is not available
	}

	public void propertyChange(PropertyChangeEvent event) {
		String prop = event.getPropertyName();
		if (prop.equals(MODEL_DATA_CHANGED)) {
			refreshGraphicalViewer();
			isPageModified = true;
			firePropertyChange(PROP_DIRTY);
		} else if (prop.equals(MODEL_DATA_LOADED)) {
			refreshGraphicalViewer();
		}
	}

	private void refreshGraphicalViewer() {
		if (graphicalViewer.getContents() == null)
			return;

		// refresh size of all feature figures
		graphicalViewer.getContents().refresh();

		// layout all features
		Point size = graphicalViewer.getControl().getSize();
		layoutManager.setControlSize(size.x, size.y);
		layoutManager.layout(featureModel);

		// refresh position of all feature figures
		graphicalViewer.getContents().refresh();

		try {
			// saveEditorContentsAsImage(graphicalViewer,
			// featureProject.getProject().getFile("model.png").getRawLocation().toOSString(),
			// SWT.IMAGE_PNG);
			// saveImage(featureProject.getProject().getFile("model.png"),
			// SWT.IMAGE_PNG);
			// ImageSaveUtil.createScreenshot(featureProject.getProject().getFile("model.png"),
			// SWT.IMAGE_PNG, graphicalViewer);
		} catch (Exception e) {
			e.printStackTrace();
		}
		// final Image image = new Image(null, size.x, size.y);
		// graphicalViewer.getControl().addPaintListener(new PaintListener() {
		// @Override
		// public void paintControl(PaintEvent e) {
		// e.gc.copyArea(image, 0, 0);
		// // image.getImageData()
		// }
		// });
	}

	// /**
	// * Saves an encoded image from this viewer.
	// *
	// * @param format one of SWT.IMAGE_BMP, SWT.IMAGE_BMP_RLE, SWT.IMAGE_GIF
	// * SWT.IMAGE_ICO, SWT.IMAGE_JPEG or SWT.IMAGE_PNG
	// * @return the bytes of an encoded image for the specified viewer
	// */
	// public void saveImage(final IFile file, final int format) {
	// WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
	// public void execute(final IProgressMonitor monitor)
	// throws CoreException {
	// try {
	// if (file.exists()) {
	// file.setContents(new ByteArrayInputStream(
	// createImage(format)), true, false, monitor);
	// } else {
	// file.create(new ByteArrayInputStream(
	// createImage(format)), true, monitor);
	// }
	// } catch (CoreException e) {
	// ErrorDialog.openError(getSite().getShell(),
	// "Editor.SaveError.title", "Editor.SaveError.text",
	// e.getStatus());
	// }
	// }
	// };
	//
	// Shell shell = getSite().getWorkbenchWindow().getShell();
	// try {
	// new ProgressMonitorDialog(shell).run(false, true, op);
	// } catch (InvocationTargetException e) {
	// e.printStackTrace();
	// } catch (InterruptedException e) {
	// e.printStackTrace();
	// }
	// }
	//
	// /**
	// * Returns the bytes of an encoded image from this viewer.
	// *
	// * @param format one of SWT.IMAGE_BMP, SWT.IMAGE_BMP_RLE, SWT.IMAGE_GIF
	// * SWT.IMAGE_ICO, SWT.IMAGE_JPEG or SWT.IMAGE_PNG
	// * @return the bytes of an encoded image for the specified viewer
	// */
	// public byte[] createImage(int format) {
	// ByteArrayOutputStream result = new ByteArrayOutputStream();
	//
	// Device device = graphicalViewer.getControl().getDisplay();
	// LayerManager lm = (LayerManager)
	// graphicalViewer.getEditPartRegistry().get(LayerManager.ID);
	// IFigure figure =
	// lm.getLayer(LayerConstants.CONNECTION_LAYER);//.PRINTABLE_LAYERS);
	// // figure = ((ModelEditPart)
	// graphicalViewer.getEditPartRegistry().get(featureModel)).getFigure();
	// figure = ((FeatureEditPart)
	// graphicalViewer.getEditPartRegistry().get(featureModel.getRoot())).getFigure().getParent();
	// Rectangle r = figure.getClientArea();
	//
	// Image image = null;
	// GC gc = null;
	// Graphics g = null;
	// try {
	// System.out.println(r);
	// image = new Image(device, r.width, r.height);
	// gc = new GC(image);
	// g = new SWTGraphics(gc);
	// g.translate(-r.x, -r.y);
	//
	// figure.paint(g);
	//
	// ImageLoader imageLoader = new ImageLoader();
	// imageLoader.data = new ImageData[] { image.getImageData() };
	// imageLoader.save(result, format);
	// } finally {
	// if (g != null) {
	// g.dispose();
	// }
	// if (gc != null) {
	// gc.dispose();
	// }
	// if (image != null) {
	// image.dispose();
	// }
	// }
	// return result.toByteArray();
	// }

	// private static void saveEditorContentsAsImage(GraphicalViewer viewer,
	// String saveFilePath, int format)
	// {
	// /* 1. First get the figure whose visuals we want to save as image.
	// * So we would like to save the rooteditpart which actually hosts all the
	// printable layers.
	// *
	// * NOTE: ScalableRootEditPart manages layers and is registered
	// graphicalviewer's editpartregistry with
	// * the key LayerManager.ID ... well that is because ScalableRootEditPart
	// manages all layers that
	// * are hosted on a FigureCanvas. Many layers exist for doing different
	// things */
	// ScalableFreeformRootEditPart rootEditPart =
	// (ScalableFreeformRootEditPart)viewer.getEditPartRegistry().get(LayerManager.ID);
	// IFigure rootFigure =
	// ((LayerManager)rootEditPart).getLayer(LayerConstants.PRINTABLE_LAYERS);//rootEditPart.getFigure();
	// Rectangle rootFigureBounds = rootFigure.getBounds();
	//		
	// /* 2. Now we want to get the GC associated with the control on which all
	// figures are
	// * painted by SWTGraphics. For that first get the SWT Control associated
	// with the viewer on which the
	// * rooteditpart is set as contents */
	// Control figureCanvas = viewer.getControl();
	// GC figureCanvasGC = new GC(figureCanvas);
	//		
	// /* 3. Create a new Graphics for an Image onto which we want to paint
	// rootFigure */
	// Image img = new Image(null, rootFigureBounds.width,
	// rootFigureBounds.height);
	// GC imageGC = new GC(img);
	// // imageGC.setBackground(figureCanvasGC.getBackground());
	// // imageGC.setForeground(figureCanvasGC.getForeground());
	// // imageGC.setFont(figureCanvasGC.getFont());
	// // imageGC.setLineStyle(figureCanvasGC.getLineStyle());
	// // imageGC.setLineWidth(figureCanvasGC.getLineWidth());
	// // imageGC.setXORMode(figureCanvasGC.getXORMode());
	// Graphics imgGraphics = new SWTGraphics(imageGC);
	//		
	// /* 4. Draw rootFigure onto image. After that image will be ready for save
	// */
	// rootFigure.paint(imgGraphics);
	//		
	// /* 5. Save image */
	// ImageData[] imgData = new ImageData[1];
	// imgData[0] = img.getImageData();
	//		
	// ImageLoader imgLoader = new ImageLoader();
	// imgLoader.data = imgData;
	// imgLoader.save(saveFilePath, format);
	//		
	// /* release OS resources */
	// figureCanvasGC.dispose();
	// imageGC.dispose();
	// img.dispose();
	// }

	// public static boolean save(IEditorPart editorPart, GraphicalViewer
	// viewer, String saveFilePath, int format)
	// {
	// Assert.isNotNull(editorPart, "null editorPart passed to
	// ImageSaveUtil::save");
	// Assert.isNotNull(viewer, "null viewer passed to ImageSaveUtil::save");
	// Assert.isNotNull(saveFilePath, "null saveFilePath passed to
	// ImageSaveUtil::save");
	//		
	// // if( format != SWT.IMAGE_BMP && format != SWT.IMAGE_JPEG && format !=
	// SWT.IMAGE_ICO )
	// // throw new IllegalArgumentException("Save format not supported");
	//				
	// try {
	// saveEditorContentsAsImage(viewer, saveFilePath, format);
	// } catch (Exception ex) {
	// ex.printStackTrace();
	// // MessageDialog.openError(editorPart.getEditorSite().getShell(), "Save
	// Error", "Could not save editor contents");
	// return false;
	// }
	//			
	// return true;
	// }
	//	
	// public static boolean save(IEditorPart editorPart, GraphicalViewer
	// viewer)
	// {
	// Assert.isNotNull(editorPart, "null editorPart passed to
	// ImageSaveUtil::save");
	// Assert.isNotNull(viewer, "null viewer passed to ImageSaveUtil::save");
	//		
	// String saveFilePath = getSaveFilePath(editorPart, viewer, -1);
	// if( saveFilePath == null ) return false;
	//		
	// int format = SWT.IMAGE_JPEG;
	// if( saveFilePath.endsWith(".jpeg") )
	// format = SWT.IMAGE_JPEG;
	// else if( saveFilePath.endsWith(".bmp") )
	// format = SWT.IMAGE_BMP;
	// else if( saveFilePath.endsWith(".ico") )
	// format = SWT.IMAGE_ICO;
	//			
	// return save(editorPart, viewer, saveFilePath, format);
	// }
	//	
	// private static String getSaveFilePath(IEditorPart editorPart,
	// GraphicalViewer viewer, int format)
	// {
	// FileDialog fileDialog = new
	// FileDialog(editorPart.getEditorSite().getShell(), SWT.SAVE);
	//		
	// String[] filterExtensions = new String[] {"*.jpeg", "*.bmp", "*.ico"/*,
	// "*.png", "*.gif"*/};
	// if( format == SWT.IMAGE_BMP )
	// filterExtensions = new String[] {"*.bmp"};
	// else if( format == SWT.IMAGE_JPEG )
	// filterExtensions = new String[] {"*.jpeg"};
	// else if( format == SWT.IMAGE_ICO )
	// filterExtensions = new String[] {"*.ico"};
	// fileDialog.setFilterExtensions(filterExtensions);
	//		
	// return fileDialog.open();
	// }

	@SuppressWarnings("unchecked")
	@Override
	public Object getAdapter(Class adapter) {
		if (IGotoMarker.class.equals(adapter)) {
			if (getActivePage() == graphicalViewerIndex)
				setActivePage(textEditorIndex);
		}
		return super.getAdapter(adapter);
	}

	@Override
	public int getActivePage() {
		return super.getActivePage();
	}

	public ITextEditor getSourceEditor() {
		return textEditor;
	}

	public FeatureModel getFeatureModel() {
		return featureModel;
	}

	public GrammarFile getGrammarFile() {
		return grammarFile;
	}

}
