package featureide_ui.editors.grammar;

import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.part.EditorActionBarContributor;
import org.eclipse.ui.texteditor.ITextEditor;

import featureide_ui.editors.GrammarEditor;
import featureide_ui.editors.grammar.actions.AndAction;
import featureide_ui.editors.grammar.actions.CreateCompoundAction;
import featureide_ui.editors.grammar.actions.CreateLayerAction;
import featureide_ui.editors.grammar.actions.DeleteAction;
import featureide_ui.editors.grammar.actions.AlternativeAction;
import featureide_ui.editors.grammar.actions.MandantoryAction;
import featureide_ui.editors.grammar.actions.OrAction;

public class GrammarEditorContributor extends EditorActionBarContributor {

	private static final String[] DIAGRAM_ACTION_IDS = { CreateLayerAction.ID,
			CreateCompoundAction.ID, DeleteAction.ID, MandantoryAction.ID,
			AndAction.ID, OrAction.ID, AlternativeAction.ID,
			ActionFactory.UNDO.getId(), ActionFactory.REDO.getId(),
			ActionFactory.CUT.getId(), ActionFactory.COPY.getId(),
			ActionFactory.PASTE.getId(), ActionFactory.SELECT_ALL.getId(),
			ActionFactory.FIND.getId(), IDEActionFactory.BOOKMARK.getId() };

	private static final String[] TEXTEDITOR_ACTION_IDS = {
			ActionFactory.DELETE.getId(), ActionFactory.UNDO.getId(),
			ActionFactory.REDO.getId(), ActionFactory.CUT.getId(),
			ActionFactory.COPY.getId(), ActionFactory.PASTE.getId(),
			ActionFactory.SELECT_ALL.getId(), ActionFactory.FIND.getId(),
			IDEActionFactory.BOOKMARK.getId() };

	@Override
	public void setActiveEditor(IEditorPart targetEditor) {
		GrammarEditor editor = (GrammarEditor) targetEditor;
		setActivePage(editor, editor.getActivePage());
	}

	public void setActivePage(GrammarEditor editor, int pageIndex) {
		IActionBars actionBars = getActionBars();
		if (actionBars != null) {
			switch (pageIndex) {
			case 0:
				hookGlobalDiagramActions(editor, actionBars);
				break;
			case 1:
				hookGlobalTextActions(editor, actionBars);
				break;
			}
			actionBars.updateActionBars();
		}
	}

	private void hookGlobalDiagramActions(GrammarEditor editor,
			IActionBars actionBars) {
		for (int i = 0; i < DIAGRAM_ACTION_IDS.length; i++) {
			actionBars.setGlobalActionHandler(DIAGRAM_ACTION_IDS[i], editor
					.getDiagramAction(DIAGRAM_ACTION_IDS[i]));
		}
	}

	private void hookGlobalTextActions(GrammarEditor editor,
			IActionBars actionBars) {
		ITextEditor textEditor = editor.getSourceEditor();
		for (int i = 0; i < TEXTEDITOR_ACTION_IDS.length; i++) {
			actionBars.setGlobalActionHandler(TEXTEDITOR_ACTION_IDS[i],
					textEditor.getAction(TEXTEDITOR_ACTION_IDS[i]));
		}
	}

	// private LabelRetargetAction retargetDeleteAction = new
	// LabelRetargetAction(
	// ActionFactory.DELETE.getId(), "Delete");
	//
	// @Override
	// public void init(IActionBars bars, IWorkbenchPage page) {
	// super.init(bars, page);
	// page.addPartListener(retargetDeleteAction);
	// }

	// @Override
	// public void contributeToMenu(IMenuManager menuManager) {
	// IMenuManager menu = new MenuManager("Feature");
	// menuManager
	// .prependToGroup(IWorkbenchActionConstants.MB_ADDITIONS, menu);
	// menu.add(retargetDeleteAction);
	// }

	// @Override
	// public void contributeToToolBar(IToolBarManager manager) {
	// manager.add(new Separator());
	// manager.add(retargetDeleteAction);
	// }

	// @Override
	// public void dispose() {
	// getPage().removePartListener(retargetDeleteAction);
	// super.dispose();
	// }

}
