package featureide_ui.editors.grammar.actions;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.gef.ui.parts.GraphicalViewerImpl;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_ui.editors.grammar.editparts.FeatureEditPart;
import featureide_ui.editors.grammar.editparts.ModelEditPart;

public class CreateCompoundAction extends Action {

	public static String ID = "featureide.createcompound";

	private static ImageDescriptor createImage = PlatformUI.getWorkbench()
	.getSharedImages().getImageDescriptor(
			ISharedImages.IMG_TOOL_NEW_WIZARD);

	private final FeatureModel featureModel;

	private ISelectionChangedListener listener = new ISelectionChangedListener() {
		public void selectionChanged(SelectionChangedEvent event) {
			IStructuredSelection selection = (IStructuredSelection) event
					.getSelection();
			setEnabled(isValidSelection(selection));
		}
	};

	public CreateCompoundAction(GraphicalViewerImpl viewer, FeatureModel featureModel) {
		super("Create compound", createImage);
		this.featureModel = featureModel;
		setEnabled(false);
		viewer.addSelectionChangedListener(listener);
	}

	@Override
	public void run() {
		int number = 0;
		while (featureModel.getFeatureNames().contains("NewCompound" + ++number));
		Feature newCompound = new Feature("NewCompound" + number);
		
		if (parent != null) {
			newCompound.setAND(parent.isAnd());
			newCompound.setMultiple(parent.isMultiple());
			
			LinkedList<Feature> newChildren = new LinkedList<Feature>();
			for (Feature feature : parent.getChildren())
				if (selectedFeatures.contains(feature)) {
					if (newCompound.isLayer()) 
						newChildren.add(newCompound);
					newCompound.addChild(feature);
				}
				else
					newChildren.add(feature);
			parent.setChildren(newChildren);

			featureModel.addFeature(newCompound);
		}
		else {
			newCompound.addChild(selectedFeatures.getFirst());
			featureModel.addFeature(newCompound);
			featureModel.setRoot(newCompound);
		}

		featureModel.handleModelDataChanged();
		
		//needed when the user creates a compound feature again on the same selection
		parent = newCompound;
	}
	
	private Feature parent = null;
	
	private LinkedList<Feature> selectedFeatures = new LinkedList<Feature>();
	
	@SuppressWarnings("unchecked")
	private boolean isValidSelection(IStructuredSelection selection) {
		// check empty selection (i.e. ModelEditPart is selected)
		if (selection.size() == 1
				&& selection.getFirstElement() instanceof ModelEditPart)
			return false;

		// check that selected features have the same parent
		selectedFeatures.clear();
		Iterator iter = selection.iterator();
		while (iter.hasNext()) {
			Object editPart = iter.next();
			if (!(editPart instanceof FeatureEditPart))
				continue;
			Feature feature = ((FeatureEditPart) editPart).getFeatureModel();
			if (selectedFeatures.isEmpty())
				parent = feature.getParent();
			else if (parent != feature.getParent())
					return false;
			selectedFeatures.add(feature);
		}
		return !selectedFeatures.isEmpty();
	}
	
}
