package featureide_ui.editors.grammar.actions;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.gef.ui.parts.GraphicalViewerImpl;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;

import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_ui.editors.grammar.editparts.FeatureEditPart;
import featureide_ui.editors.grammar.editparts.ModelEditPart;

public class DeleteAction extends Action {

	public static String ID = ActionFactory.DELETE.getId();

	private static ImageDescriptor deleteImage = PlatformUI.getWorkbench()
			.getSharedImages()
			.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE);

	private final GraphicalViewerImpl viewer;

	private final FeatureModel featureModel;

	private ISelectionChangedListener listener = new ISelectionChangedListener() {
		public void selectionChanged(SelectionChangedEvent event) {
			IStructuredSelection selection = (IStructuredSelection) event
					.getSelection();
			setEnabled(isValidSelection(selection));
		}
	};

	public DeleteAction(GraphicalViewerImpl viewer, FeatureModel featureModel) {
		super("Delete", deleteImage);
		this.viewer = viewer;
		this.featureModel = featureModel;
		setEnabled(false);
		viewer.addSelectionChangedListener(listener);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void run() {
		IStructuredSelection selection = (IStructuredSelection) viewer
				.getSelection();
		Iterator iter = selection.iterator();
		while (iter.hasNext()) {
			Object editPart = iter.next();
			if (!(editPart instanceof FeatureEditPart))
				continue;
			Feature feature = ((FeatureEditPart) editPart)
					.getFeatureModel();
			if (feature == featureModel.getRoot())
				featureModel.replaceRoot(featureModel.getRoot().removeLastChild());
			else
				featureModel.deleteFeature(feature);
		}
		featureModel.handleModelDataChanged();
	}

	@SuppressWarnings("unchecked")
	private boolean isValidSelection(IStructuredSelection selection) {
		// check empty selection (i.e. ModelEditPart is selected)
		if (selection.size() == 1
				&& selection.getFirstElement() instanceof ModelEditPart)
			return false;

		// check that at least one internal and one external node remains
		Feature root = featureModel.getRoot();
		LinkedList<Feature> features = new LinkedList<Feature>(featureModel.getFeatures());
		Iterator iter = selection.iterator();
		while (iter.hasNext()) {
			Object editPart = iter.next();
			if (!(editPart instanceof FeatureEditPart))
				continue;
			Feature feature = ((FeatureEditPart) editPart).getFeatureModel();
			if (feature == root && root.getChildrenCount() != 1)
				return false;
			features.remove(feature);
		}
		for (Feature layer : features)
			if (layer.isLayer())
				for (Feature compound : features)
					if (compound.isCompound())
						return true;
		return false;
	}
}
