package featureide_ui.editors.grammar.actions;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.gef.ui.parts.GraphicalViewerImpl;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;

import featureide_core.model.Feature;
import featureide_core.model.PropertyConstants;
import featureide_ui.editors.grammar.editparts.ConnectionEditPart;
import featureide_ui.editors.grammar.editparts.FeatureEditPart;

public abstract class SingleSelectionAction extends Action implements
		PropertyChangeListener, PropertyConstants {

	private ISelectionChangedListener listener = new ISelectionChangedListener() {
		public void selectionChanged(SelectionChangedEvent event) {
			IStructuredSelection selection = (IStructuredSelection) event.getSelection();
			SingleSelectionAction.this
					.selectionChanged(isValidSelection(selection));
		}
	};

	private GraphicalViewerImpl viewer;

	protected Feature feature;
	
	protected boolean connectionSelected;

	public SingleSelectionAction(String text, GraphicalViewerImpl viewer) {
		super(text);
		this.viewer = viewer;
		setEnabled(false);
		viewer.addSelectionChangedListener(listener);
	}

	private boolean isOneFeatureSelected(IStructuredSelection selection) {
		return selection.size() == 1
				&& (selection.getFirstElement() instanceof FeatureEditPart || selection
						.getFirstElement() instanceof ConnectionEditPart);
	}

	public Feature getSelectedFeature() {
		IStructuredSelection selection = (IStructuredSelection) viewer
				.getSelection();
		Object part = selection.getFirstElement();
		connectionSelected = part instanceof ConnectionEditPart;
		if (connectionSelected)
			return ((ConnectionEditPart) part).getConnectionModel().getTarget();
		return ((FeatureEditPart) part).getFeatureModel();
	}

	private void selectionChanged(boolean oneSelected) {
		if (feature != null)
			feature.removeListener(this);
		if (oneSelected) {
			feature = getSelectedFeature();
			feature.addListener(this);
			updateProperties();
		} else {
			feature = null;
			setEnabled(false);
			setChecked(false);
		}
	}

	protected boolean isValidSelection(IStructuredSelection selection) {
		return isOneFeatureSelected(selection);
	}

	protected abstract void updateProperties();

	public void propertyChange(PropertyChangeEvent event) {
		String prop = event.getPropertyName();
		if (prop.equals(CHILDREN_CHANGED) || prop.equals(MANDANTORY_CHANGED)
				|| prop.equals(PARENT_CHANGED)) {
			updateProperties();
		}
	}

}
