package featureide_ui.editors.grammar.commands;

import org.eclipse.gef.commands.Command;

import featureide_core.model.FeatureModel;

public class FeatureRenamingCommand extends Command {
	
	private final FeatureModel featureModel;

	private final String oldName;
	
	private final String newName;

	public FeatureRenamingCommand(FeatureModel featureModel, String oldName, String newName) {
		super("Renaming feature " + oldName);
		this.featureModel = featureModel;
		this.oldName = oldName;
		this.newName = newName.trim();
	}
	
	@Override
	public boolean canExecute() {
		if (newName == null)
			return false;
		if (featureModel.getFeatureNames().contains(newName))
			return false;
		return isValidJavaIdentifier(newName);
	}
	
	@Override
	public void execute() {
		featureModel.renameFeature(oldName, newName);
		featureModel.handleModelDataChanged();
	}
	
	@Override
	public void undo() {
		featureModel.renameFeature(newName, oldName);
		featureModel.handleModelDataChanged();
	}
	
	public static boolean isValidJavaIdentifier(String s) {
		if (s == null)
			return false;
		final int len = s.length();
		if (len == 0 || !Character.isJavaIdentifierStart(s.charAt(0)))
			return false;
		for (int i = 1; i < len; i++) {
			if (!Character.isJavaIdentifierPart(s.charAt(i)))
				return false;
		}
		return true;
	}



}
