package featureide_ui.editors.grammar.editparts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.viewers.TextCellEditor;

import featureide_core.model.FeatureConnection;
import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_core.model.PropertyConstants;
import featureide_ui.editors.FeatureUIHelper;
import featureide_ui.editors.grammar.commands.renaming.FeatureCellEditorLocator;
import featureide_ui.editors.grammar.commands.renaming.FeatureLabelEditManager;
import featureide_ui.editors.grammar.figures.FeatureFigure;
import featureide_ui.editors.grammar.policies.FeatureDirectEditPolicy;

/**
 * An editpart for features. It implements the <code>NodeEditPart</code> that
 * the models of features can provide connection anchors.
 * 
 * @author Thomas Thm
 *
 */
public class FeatureEditPart extends AbstractGraphicalEditPart implements NodeEditPart, PropertyConstants, PropertyChangeListener {
	
	public FeatureEditPart(Feature feature) {
		super();
		setModel(feature);
	}
	
	public Feature getFeatureModel() {
		return (Feature) getModel();
	}
	
	public FeatureFigure getFeatureFigure() {
		return (FeatureFigure) getFigure();
	}
	
	@Override
	protected IFigure createFigure() {
		FeatureFigure figure = new FeatureFigure(getFeatureModel());
		return figure;
	}

	@Override
	protected void createEditPolicies() {
		FeatureModel featureModel = ((ModelEditPart) getParent()).getFeatureModel();
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new FeatureDirectEditPolicy(featureModel, getFeatureModel()));
	}
	
	private DirectEditManager manager;
	
	@Override
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) {
			if (manager == null)
				manager = new FeatureLabelEditManager(this, TextCellEditor.class,
						new FeatureCellEditorLocator((FeatureFigure) getFigure()));
			manager.show();
		}
		else if (request.getType() == RequestConstants.REQ_OPEN) {
			Feature feature = getFeatureModel();
			feature.setMandatory(!feature.isMandatory());

			ModelEditPart parent = (ModelEditPart) getParent();
			FeatureModel featureModel = parent.getFeatureModel();
			featureModel.handleModelDataChanged();
		}
	}

	@Override
	protected List<FeatureConnection> getModelSourceConnections() {
		return ((Feature) getModel()).getSourceConnections();
	}
	
	@Override
	protected List<FeatureConnection> getModelTargetConnections() {
		return ((Feature) getModel()).getTargetConnections();
	}
	
	public ConnectionAnchor getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart connection) {
		return ((FeatureFigure) figure).getSourceAnchor();
	}

	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return ((FeatureFigure) figure).getSourceAnchor();
	}

	public ConnectionAnchor getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart connection) {
		return ((FeatureFigure) figure).getTargetAnchor();
	}

	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return ((FeatureFigure) figure).getTargetAnchor();
	}

	@Override
	public void activate() {
		getFeatureModel().addListener(this);
		super.activate();
	}
	
	@Override
	public void deactivate() {
		super.deactivate();
		getFeatureModel().removeListener(this);
	}

	/*
	 * (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	@SuppressWarnings("unchecked")
	public void propertyChange(PropertyChangeEvent event) {
		String prop = event.getPropertyName();
		if (prop.equals(LOCATION_CHANGED)) {
			getFeatureFigure().setLocation((Point) event.getNewValue());
		}
		else if (prop.equals(CHILDREN_CHANGED)) {
			getFeatureFigure().setCompound(getFeatureModel().isCompound());
			for (FeatureConnection connection : getFeatureModel().getTargetConnections()) {
				Map registry = getViewer().getEditPartRegistry();
				ConnectionEditPart connectionEditPart = (ConnectionEditPart) registry.get(connection);
				if (connectionEditPart != null) {
					connectionEditPart.refreshSourceDecoration();
					connectionEditPart.refreshTargetDecoration();
					connectionEditPart.refreshToolTip();
				}
			}
		}
		else if (prop.equals(NAME_CHANGED)) {
			getFeatureFigure().setName(getFeatureModel().getName());
			FeatureUIHelper.setSize(getFeatureModel(),getFeatureFigure().getSize());
		}
	}
	
}
