package featureide_ui.editors.grammar.editparts;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import featureide_core.model.Constraint;
import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_ui.editors.grammar.GUIDefaults;
import featureide_ui.editors.grammar.policies.ModelLayoutEditPolicy;

/**
 * The main editpart that has all <code>FeatureEditPart</code>s as children.
 * Notice that Draw2D calls a figure child of another when its drawn within
 * the parent figure.
 * 
 * @author Thomas Thm
 *
 */
public class ModelEditPart extends AbstractGraphicalEditPart implements GUIDefaults {
	
	public ModelEditPart(FeatureModel featureModel) {
		super();
		setModel(featureModel);
	}
	
	public FeatureModel getFeatureModel() {
		return (FeatureModel) getModel();
	}
	
	protected IFigure createFigure() {
		Figure fig = new FreeformLayer();
		fig.setLayoutManager(new FreeformLayout());
		fig.setBorder(new MarginBorder(5));
		return fig;
	}
	
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new ModelLayoutEditPolicy(getFeatureModel()));
	}
	
	@SuppressWarnings("unchecked")
	@Override
	protected List getModelChildren() {
		LinkedList<Object> list = new LinkedList<Object>();
		addFeatures(getFeatureModel().getRoot(), list);
		addConstraints(getFeatureModel().getConstraints(), list);
		return list;
	}

	private void addFeatures(Feature feature, List<Object> list) {
		if (feature == null)
			return;
		list.add(feature);
		for (Feature child : feature.getChildren())
			addFeatures(child, list);
	}
	
	private void addConstraints(List<Constraint> constraints, LinkedList<Object> list) {
		for (int i = 0; i < constraints.size(); i++)
			list.add(constraints.get(i));
	}

}
