package featureide_ui.editors.grammar.figures;

import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.RotatableDecoration;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import featureide_core.model.Feature;
import featureide_ui.editors.FeatureUIHelper;
import featureide_ui.editors.grammar.GUIDefaults;

/**
 * A decoration for a feature connection that indicates relation of its
 * children (AND, OR, XOR).
 * 
 * @author Thomas Thm
 *
 */
public class RelationDecoration extends Ellipse implements RotatableDecoration, GUIDefaults {
	
	private boolean fill;
	
	private Point referencePoint;

	private Feature lastChild;

	public RelationDecoration(boolean fill, Feature lastChild) {
		super();
		this.fill = fill;
		this.lastChild = lastChild;
		setForegroundColor(DECORATOR_FOREGROUND);
		setBackgroundColor(DECORATOR_FOREGROUND);
		setSize(TARGET_ANCHOR_DIAMETER, TARGET_ANCHOR_DIAMETER/2);
	}
	
	@Override
	public void setLocation(Point p) {
		super.setLocation(p.translate(-TARGET_ANCHOR_DIAMETER/2, 0));
	}
	
	public void setReferencePoint(Point p) {
		referencePoint = p;
	}
	
	@Override
	protected void fillShape(Graphics graphics) {
		if (!fill)
			return;
		
		Rectangle r = calculateRectangle();

		int angle1 = ABSTRACT_FEATURES ? 180 : calculateAngle(r.getCenter(), referencePoint);
		int angle2 = ABSTRACT_FEATURES ? 360 : calculateAngle(r.getCenter(), FeatureUIHelper.getSourceLocation(lastChild));

		if (angle1 + 1 < angle2)
			graphics.fillArc(r, angle1, angle2 - angle1);
	}
	
	@Override
	protected void outlineShape(Graphics graphics) {
		if (fill)
			return;
		
		Rectangle r = calculateRectangle();
		r.shrink(lineWidth, lineWidth);

		int angle1 = ABSTRACT_FEATURES ? 180 : calculateAngle(r.getCenter(), referencePoint);
		int angle2 = ABSTRACT_FEATURES ? 360 : calculateAngle(r.getCenter(), FeatureUIHelper.getSourceLocation(lastChild));

		if (angle1 + 1 < angle2)
			graphics.drawArc(r, angle1, angle2 - angle1);
	}
	
	private Rectangle calculateRectangle() {
		Rectangle r = Rectangle.SINGLETON;
		r.setBounds(getBounds());
		r.y -= TARGET_ANCHOR_DIAMETER/2;
		r.height = TARGET_ANCHOR_DIAMETER;
		r.shrink(lineWidth + 1, lineWidth + 1);
		return r;
	}

	private int calculateAngle(Point point, Point referencePoint) {
		int dx = referencePoint.x - point.x;
		int dy = referencePoint.y - point.y;
		return 360 - (int) Math.round(Math.atan2(dy, dx)/Math.PI*180);
	}

}
