package featureide_ui.editors.grammar.layouts;

import java.util.LinkedList;

import org.eclipse.draw2d.geometry.Point;

import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_ui.editors.FeatureUIHelper;

/**
 * Layouts the features at the feature diagram using a breadth first search.
 * 
 * @author Thomas Thm
 *
 */
public class BreadthFirstLayout extends FeatureLayoutManager {

	private int controlWidth = 0;

	@Override
	public void setControlSize(int width, int height) {
		controlWidth  = width;
	}
	
	public void layout(FeatureModel featureModel) {
		layout(featureModel.getRoot());
	}
	
	private void layout(Feature root) {
		if (root == null)
			return;
		LinkedList<Feature> list = new LinkedList<Feature>();
		list.add(root);

		int yOffset = LAYOUT_MARGIN_Y;
		while (!list.isEmpty()) {
			//center the features of the level
			int width = 2 * LAYOUT_MARGIN_X - FEATURE_SPACE_X;
			for (Feature feature : list)
				width += FeatureUIHelper.getSize(feature).width + FEATURE_SPACE_X;
			int xOffset = controlWidth / 2 - width / 2;

			//set location of each feature at this level
			int levelSize = list.size();
			for (int i = 0; i < levelSize; i++) {
				Feature feature = list.removeFirst();
				FeatureUIHelper.setLocation(feature,new Point(xOffset, yOffset));
				xOffset += FeatureUIHelper.getSize(feature).width + FEATURE_SPACE_X;
				
				//add the features children
				for (Feature child : feature.getChildren())
					list.add(child);
			}
			yOffset += FEATURE_SPACE_Y;
		}
	}
	
}
