package featureide_ui.editors.grammar.layouts;

import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;

import featureide_core.model.Constraint;
import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_ui.editors.FeatureUIHelper;

/**
 * Layouts the features at the feature diagram using a reverse level order search.
 * 
 * @author Thomas Thm
 *
 */
public class LevelOrderLayout extends FeatureLayoutManager {
	
//	private int myLevel = 0;

	private int controlWidth = 0;
	
	private int featureDiagramBottom = 0;

	@Override
	public void setControlSize(int width, int height) {
		controlWidth = width;
	}
	
	public void layout(FeatureModel featureModel) {
		layout(featureModel.getRoot());
		layout(featureModel.getConstraints());
	}
	
	private void layout(Feature root) {
		Vector<Vector<Feature>> levels = calculateLevels(root);

//		myLevel++;
		for (int i = levels.size() - 1; i >= 0; i--) {
			int y = LAYOUT_MARGIN_Y + FEATURE_SPACE_Y * i;
			for (Feature feature : levels.get(i))
				FeatureUIHelper.setLocation(feature,new Point(0, y));

//			if (levels.size() - i <= myLevel) {
			for (Feature feature : levels.get(i))
				if (feature.isCompound()) {
					centerAboveChildren(feature);
				}

			Feature lastFeature = null;
			int moveWidth = 0;
			for (int j = 0; j < levels.get(i).size(); j++) {
				Feature feature = levels.get(i).get(j);
				if (!feature.isCompound())
					nextToSibling(feature, lastFeature);
				else {
					if (lastFeature != null)
						moveWidth = Math.max(moveWidth, FeatureUIHelper.getBounds(lastFeature).right() + FEATURE_SPACE_X - FeatureUIHelper.getLocation(feature).x);
					if (moveWidth > 0)
						moveTree(feature, moveWidth);
					int width = FEATURE_SPACE_X;
					int l = 0;
					int space = 0;
					boolean right = true;
					for (int k = j - 1; k >= 0; k--) {
						Feature sibling = levels.get(i).get(k);
						if (sibling.getParent() != feature.getParent()) {
							l = k + 1;
							break;
						}
						if (sibling.isCompound()) {
							l = k + 1;
							right = false;
							space = FeatureUIHelper.getBounds(feature).x - FeatureUIHelper.getBounds(sibling).right() - width;
							break;
						}
						width += FeatureUIHelper.getSize(sibling).width + FEATURE_SPACE_X;
					}
					if (right)
						space = FeatureUIHelper.getBounds(feature).x - (FeatureUIHelper.getBounds(levels.get(i).get(l)).x - FEATURE_SPACE_X) - width;
					for (int k = l; k < j; k++) {
						Feature sibling = levels.get(i).get(k);
						if (right)
							moveTree(sibling, space);
						else
							moveTree(sibling, space * (k - l + 1) / (j - l + 1));
					}
				}
				lastFeature = feature;
			}
//			}
		}
		
		int newX = (controlWidth - FeatureUIHelper.getBounds(root).width) / 2;
		moveTree(root, newX - FeatureUIHelper.getLocation(root).x);
		
		featureDiagramBottom = LAYOUT_MARGIN_Y + FEATURE_SPACE_Y * (levels.size() - 1);
	}

	private void layout(List<Constraint> constraints) {
		int y = featureDiagramBottom + CONSTRAINT_SPACE_Y;
		for (int i = 0; i < constraints.size(); i++) {
			Constraint constraint = constraints.get(i);
			Dimension size = FeatureUIHelper.getSize(constraint);
			int x = (controlWidth - size.width) / 2;
			FeatureUIHelper.setLocation(constraint,new Point(x,y));
			y += size.height;
		}		
	}

	private Vector<Vector<Feature>> calculateLevels(Feature root) {
		Vector<Vector<Feature>> levels = new Vector<Vector<Feature>>();
		
		Vector<Feature> level = new Vector<Feature>();
		level.add(root);
		
		while (!level.isEmpty()) {
			levels.add(level);
			Vector<Feature> newLevel = new Vector<Feature>();
			for (Feature feature : level)
				for (Feature child : feature.getChildren())
					newLevel.add(child);
			level = newLevel;
		}
		
		return levels;
	}
	
	private void centerAboveChildren(Feature feature) {
		int minX = FeatureUIHelper.getBounds(feature.getFirstChild()).x;
		int maxX = FeatureUIHelper.getBounds(feature.getLastChild()).right();
		Point location = FeatureUIHelper.getLocation(feature);
		int x = (maxX + minX) / 2 - FeatureUIHelper.getSize(feature).width / 2;
		FeatureUIHelper.setLocation(feature,new Point(x, location.y));
	}

	private void nextToSibling(Feature feature, Feature lastFeature) {
		Point location = FeatureUIHelper.getLocation(feature);
		int x = lastFeature != null ? FeatureUIHelper.getBounds(lastFeature).right() + FEATURE_SPACE_X : 0;
		FeatureUIHelper.setLocation(feature,new Point(x, location.y));
	}

	private void moveTree(Feature feature, int deltaX) {
		Point location = FeatureUIHelper.getLocation(feature);
		FeatureUIHelper.setLocation(feature,new Point(location.x + deltaX, location.y));
		for (Feature child : feature.getChildren())
			moveTree(child, deltaX);
	}

}
