package featureide_ui.editors.grammar.policies;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYAnchor;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionRectangle;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;

import featureide_ui.editors.FeatureUIHelper;
import featureide_ui.editors.grammar.GUIDefaults;
import featureide_ui.editors.grammar.commands.FeatureDragAndDropCommand;
import featureide_ui.editors.grammar.editparts.FeatureEditPart;

public class FeatureMoveEditPolicy extends NonResizableEditPolicy implements GUIDefaults {

	private FeatureEditPart editPart;

	private ModelLayoutEditPolicy superPolicy;

	public FeatureMoveEditPolicy(FeatureEditPart editPart, ModelLayoutEditPolicy superPolicy) {
		this.editPart = editPart;
		this.superPolicy = superPolicy;
	}
	
	private Point s;
	
	private RectangleFigure r;
	
	private PolylineConnection c;

	@Override
	protected IFigure createDragSourceFeedbackFigure() {
		r = new RectangleFigure();
		FigureUtilities.makeGhostShape(r);
		r.setLineStyle(Graphics.LINE_DOT);
		r.setForegroundColor(ColorConstants.white);
		r.setBounds(getInitialFeedbackBounds());
		
		s = FeatureUIHelper.getSourceLocation(editPart.getFeatureModel());
		Point s2 = s.getCopy();
		getHostFigure().translateToAbsolute(s2);

		c = new PolylineConnection();
		c.setForegroundColor(NEW_CONNECTION_FOREGROUND);
		c.setSourceAnchor(new XYAnchor(s2));
		c.setTargetAnchor(new XYAnchor(s2));
		
		FreeformLayer l = new FreeformLayer();
		l.add(r);
		l.add(c);
		
		addFeedback(l);
		return l;
	}
	
	@Override
	protected void showChangeBoundsFeedback(ChangeBoundsRequest request) {
		//call createDragSourceFeedbackFigure on start of the move
		getDragSourceFeedbackFigure();
		
		PrecisionRectangle rect = new PrecisionRectangle(getInitialFeedbackBounds().getCopy());
		getHostFigure().translateToAbsolute(rect);
		rect.translate(request.getMoveDelta());
		rect.resize(request.getSizeDelta());
		r.translateToRelative(rect);
		r.setBounds(rect);
		
		Point s2 = s.getCopy();
		getHostFigure().translateToAbsolute(s2);
		s2.translate(request.getMoveDelta());
		c.setSourceAnchor(new XYAnchor(s2));

		FeatureDragAndDropCommand cmd = superPolicy.getConstraintCommand();
		Point location;
		if (cmd != null && cmd.getNewParent() != null) {
			location = FeatureUIHelper.getTargetLocation(cmd.getNewParent());
			getHostFigure().translateToAbsolute(location);
			c.setForegroundColor(cmd.canExecute() ? NEW_CONNECTION_FOREGROUND : VOID_CONNECTION_FOREGROUND);			
		}
		else
			location = s2;
		c.setTargetAnchor(new XYAnchor(location));
	}
	
	@Override
	protected void eraseChangeBoundsFeedback(ChangeBoundsRequest request) {
		super.eraseChangeBoundsFeedback(request);
		s = null;
		r = null;
		c = null;
	}

}
