package featureide_ui.editors.grammar.policies;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.LayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import featureide_core.model.Feature;
import featureide_core.model.FeatureModel;
import featureide_ui.editors.FeatureUIHelper;
import featureide_ui.editors.grammar.commands.FeatureDragAndDropCommand;
import featureide_ui.editors.grammar.editparts.FeatureEditPart;

public class ModelLayoutEditPolicy extends LayoutEditPolicy {

	private final FeatureModel featureModel;

	private FeatureDragAndDropCommand cmd;
	
	public ModelLayoutEditPolicy(FeatureModel featureModel) {
		super();
		this.featureModel = featureModel;
	}

	@Override
	protected EditPolicy createChildEditPolicy(EditPart child) {
		if (child instanceof FeatureEditPart)
			return new FeatureMoveEditPolicy((FeatureEditPart) child, this);
		return null;
	}
	
	@Override
	protected Command getMoveChildrenCommand(Request request) {
		cmd = null;
		if (request instanceof ChangeBoundsRequest) {
			ChangeBoundsRequest r = (ChangeBoundsRequest) request;
			if (r.getEditParts().size() != 1)
				return null;
			FeatureEditPart editPart = (FeatureEditPart) r.getEditParts().get(0);
			Feature feature = editPart.getFeatureModel();
			Rectangle bounds = FeatureUIHelper.getBounds(feature);
			bounds = r.getTransformedRectangle(bounds);
			cmd = new FeatureDragAndDropCommand(featureModel, feature, bounds.getLocation());
		}
		return cmd;
	}

	public FeatureDragAndDropCommand getConstraintCommand() {
		return cmd;
	}

	@Override
	protected Command getCreateCommand(CreateRequest request) {
		//no creation supported
		return null;
	}

}
